@@ -1,5 +1,5 @@
 ######################################################################
-## Copyright (C) 2008, Cameron Bracken <cameron.bracken@gmail.com>
+## Copyright (C) 2011, Cameron Bracken <cameron.bracken@gmail.com>
 ##             Charlie Sharpsteen <source@shaprpsteen.net>
 ##
 ## This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
 #####################################################################
 
 ######################################################################
-## Heavily relies on functions of cacheSweave but reimplements the 
+## Heavily relies on functions of cacheSweave but reimplements the
 ## Sweave driver function.
 
 pgfSweaveDriver <- function() {
@@ -28,20 +28,20 @@ pgfSweaveDriver <- function() {
        runcode = pgfSweaveRuncode,
        writedoc = pgfSweaveWritedoc,
        finish = utils::RweaveLatexFinish,
-       checkopts = pgfSweaveOptions 
+       checkopts = pgfSweaveOptions
        )
 }
 
-    
+
 source('R/cacheSweaveUnexportedFunctions.R')
 source('R/utilities.R')
 
 ## Add the 'pgf' and 'external', 'pdflatex', 'sanitize' option to the list
 pgfSweaveSetup <- function(file, syntax,
               output=NULL, quiet=FALSE, debug=FALSE, echo=TRUE,
-              eval=TRUE, split=FALSE, stylepath=TRUE, 
-              pdf=FALSE, eps=FALSE, cache=FALSE, pgf=FALSE, 
-              tikz=TRUE, external=FALSE, sanitize = FALSE, 
+              eval=TRUE, split=FALSE, stylepath=TRUE,
+              pdf=FALSE, eps=FALSE, cache=FALSE, pgf=FALSE,
+              tikz=TRUE, external=FALSE, sanitize = FALSE,
               highlight = TRUE, tidy = FALSE, tex.driver="pdflatex")
 {
     out <- utils::RweaveLatexSetup(file, syntax, output=output, quiet=quiet,
@@ -70,19 +70,23 @@ pgfSweaveSetup <- function(file, syntax,
     out[["mapFile"]] <- makeMapFileName(file)
     file.create(out[["mapFile"]])  ## Overwrite an existing file
     ## End additions [RDP]
-    
-    ## [CWB]  create a shell script with a command for each modified
-    ##      graphic 
+
+    ## [CWB]  create a shell script with a command for each modified graphic
     out[["shellFile"]] <- makeExternalShellScriptName(file)
-    out[["srcfileName"]] <- sub("\\.Rnw$", "\\.tex", file)
+    out[["srcfileName"]] <- sub("\\.[Rr]nw$", "\\.tex", file)
     out[["jobname"]] <- basename(removeExt(file))
     file.create(out[["shellFile"]])  ## Overwrite an existing file
     ######################################################################
 
+      # Avoids creation of Rplots.pdf
+    options(device = function(...) {
+        .Call("R_GD_nullDevice", PACKAGE = "grDevices")
+    })
+
     out
 }
 
-    # changes in Sweave in 2.12 make it necessary to copy this function here 
+    # changes in Sweave in 2.12 make it necessary to copy this function here
     # to register the tex.driver option, I wish there was an easier way....
 pgfSweaveOptions <- function(options)
 {
@@ -129,7 +133,7 @@ pgfSweaveOptions <- function(options)
 
     # Copied from Sweave in R version 2.12, internal chages make it necessary
     # to register the tex.driver option as a "NOLOGOPT"
-    # This function checks for the \usepackage{Sweave} line and the 
+    # This function checks for the \usepackage{Sweave} line and the
     # \pgfrealjobname line
 pgfSweaveWritedoc <- function(object, chunk)
 {
@@ -137,62 +141,72 @@ pgfSweaveWritedoc <- function(object, chunk)
 
     if(length(grep("\\usepackage[^\\}]*Sweave.*\\}", chunk)))
         object$havesty <- TRUE
-        
+
     haverealjobname <- FALSE
     if(length(grep("\\pgfrealjobname\\{.*\\}", chunk)))
         haverealjobname <- TRUE
-        
-    
+
+
     if(!object$havesty){
-    begindoc <- "^[[:space:]]*\\\\begin\\{document\\}"
-    which <- grep(begindoc, chunk)
-    if (length(which)) {
-      print(object$srcfile)
+      begindoc <- "^[[:space:]]*\\\\begin\\{document\\}"
+      which <- grep(begindoc, chunk)
+      if (length(which)) {
+        if(length(which) > 1) cat('havesty',which,'\n')
             chunk[which] <- paste("\\usepackage{", object$styfile, "}\n",
                   chunk[which], sep="")
             linesout <- linesout[c(1L:which, which, seq(from=which+1L, length.out=length(linesout)-which))]
             object$havesty <- TRUE
         }
     }
-    
+
       # add pgfrealjobname if it doesnt exist
     if(!haverealjobname){
      begindoc <- "^[[:space:]]*\\\\begin\\{document\\}"
      which <- grep(begindoc, chunk)
       # if Sweave line also does not exist
-     otherwhich <- grep("\\usepackage[^\\}]*Sweave.*\\}", chunk)
-     if(length(which) == 0) which <- otherwhich
+     #otherwhich <- grep("\\usepackage[^\\}]*Sweave.*\\}", chunk)
+     otherwhich <- grep("^[[:space:]]*\\\\usepackage\\{.*tikz.*\\}", chunk)
+     other <- FALSE
+     if(length(which) == 0){which <- otherwhich;other <- T}
      if (length(which)) {
-             chunk[which] <- paste("\\pgfrealjobname{",object$jobname,"}\n",
-                      chunk[which], sep="")
-             linesout <- linesout[c(1L:which, which, seq(from=which+1L, length.out=length(linesout)-which))]
-             object$haverealjobname <- TRUE
-         }
-     }
+     if(length(which) > 1) {cat('haverealjobname',which,'\n');browser()}
+      chunk[which] <- 
+        if(other) 
+          paste(chunk[which],"\n\\pgfrealjobname{",object$jobname,"}\n", sep="")
+        else
+          paste("\\pgfrealjobname{",object$jobname,"}\n",chunk[which], sep="")
+      linesout <- linesout[c(1L:which, which, seq(from=which+1L, length.out=length(linesout)-which))]
+      object$haverealjobname <- TRUE
+      }
+    }
      
-     # always add the syntax definitions because there is no real way to 
-     # check if a single code chunk as the option before hand. 
+     # always add the syntax definitions because there is no real way to
+     # check if a single code chunk as the option before hand.
       if (!object$haveHighlightSyntaxDef){
-                # get the latex style definitions from the highlight package
-          tf <- tempfile()
-          cat(styler('default', 'sty', styler_assistant_latex),sep='\n',file=tf)
-          cat(boxes_latex(),sep='\n',file=tf,append=T)
-          hstyle <- readLines(tf)
-                # find where to put the style definitions
+              # get the latex style definitions from the highlight package
+        tf <- tempfile()
+        cat(styler('default', 'sty', styler_assistant_latex),sep='\n',file=tf)
+        cat(boxes_latex(),sep='\n',file=tf,append=T)
+        hstyle <- readLines(tf)
+          # find where to put the style definitions
         begindoc <- "^[[:space:]]*\\\\begin\\{document\\}"
-            which <- grep(begindoc, chunk)
-            otherwhich <- grep("\\usepackage[^\\}]*Sweave.*\\}", chunk)
-            if(!length(which)) which <- otherwhich
-                # put in the style definitions before the \begin{document}
+          # first try to put it before the begin{document} command
+          # this may not work if there is a code chunk before it
+        which <- grep(begindoc, chunk)
+          # also try to put it by the Sweave style file, 
+        otherwhich <- grep("\\usepackage[^\\}]*Sweave.*\\}", chunk)
+        if(!length(which)) which <- otherwhich
+        
+              # put in the style definitions before the \begin{document}
+              # or \usepackage{Sweave}
         if(length(which)) {
-                chunk <- c(chunk[1:(which-1)],hstyle,chunk[which:length(chunk)])
-
-                linesout <- linesout[c(1L:which, which, seq(from = which + 
-                    1L, length.out = length(linesout) - which))]
+    if(length(which) > 1) cat('havehighlight',which,'\n')          
+          chunk <- c(chunk[1:(which-1)],hstyle,chunk[which:length(chunk)])
+          linesout <- linesout[c(1L:which, which, seq(from = which +
+              1L, length.out = length(linesout) - which))]
           object$haveHighlightSyntaxDef <- TRUE
         }
       }
-     
     while(length(pos <- grep(object$syntax$docexpr, chunk)))
     {
         cmdloc <- regexpr(object$syntax$docexpr, chunk[pos[1L]])
@@ -238,14 +252,16 @@ pgfSweaveWritedoc <- function(object, chunk)
 
 
 
-## This function is essentially unchanged from the original Sweave
+## [RDP] This function is essentially unchanged from the original Sweave
 ## version, except I compute the digest of the entire chunk, write out
 ## information to the map file, and use 'cacheSweaveEvalWithOpt'
 ## instead.  Note that everything in this function operates at the
 ## chunk level.  The code has been copied from R 2.5.0.
 
+## [CWB] Here are the main changes that allow for tikz graphic output and 
+## highlighting and such
 pgfSweaveRuncode <- function(object, chunk, options) {
-    
+
   if(!(options$engine %in% c("R", "S"))){
     return(object)
   }
@@ -294,30 +310,37 @@ pgfSweaveRuncode <- function(object, chunk, options) {
   on.exit(options(saveopts))
 
   SweaveHooks(options, run=TRUE)
-  # remove unwanted "#line" comments added by R 2.12
-  if(substring(chunk[1], 1, 5) == "#line") chunk <- chunk[-1]
-  
+
+    # remove unwanted "#line" directives added by R 2.12
+  removeLineJunk <- function(chunk){
+    lines <- grep("#line", chunk)
+    srclines <- attr(chunk, "srclines")
+    chunk <- chunk[-lines]
+    attr(chunk, "srclines") <- srclines[-lines]
+    chunk
+  }
+  chunk <- removeLineJunk(chunk)
+
   ## parse entire chunk block
-  chunkexps <- 
+  chunkexps <-
     if(options$tidy){
-      try(parse2(text=chunk), silent=TRUE) 
+      try(parse.tidy(text=chunk), silent=TRUE)
     }else{
       try(parse(text=chunk), silent=TRUE)
     }
+
   RweaveTryStop(chunkexps, options)
 
-  ## [CWB] Create a DB entry which is simply the digest of the text of 
-  ## the chunk so that if anything has changed the chunk will be 
-  ## recognised as having a change if ANYTHING gets changed, even white 
-  ## space. this still doesnt fix the problem of actually caching the 
-  ## plotting commands and others which do not create objects in the 
-  ## global environment when they are evaluated but at least the figures 
+  ## [CWB] Create a DB entry which is simply the digest of the text of
+  ## the chunk so that if anything has changed the chunk will be
+  ## recognised as having a change if ANYTHING gets changed, even white
+  ## space. this still doesnt fix the problem of actually caching the
+  ## plotting commands and others which do not create objects in the
+  ## global environment when they are evaluated but at least the figures
   ## will get regenerated.
-  chunkTextEvalString <- paste("chunkText <- '", 
+  chunkTextEvalString <- paste("chunkText <- '",
     digest(paste(chunk,collapse='')), "'", sep='')
   attr(chunk, "digest") <- digest(paste(chunk,collapse=''))
-  #if(substr(chunk[1],1,9)!='chunkText')
-  #  chunk <- c(chunkTextEvalString, chunk)
   ## end [CWB]
 
   ## Adding my own stuff here [RDP]
@@ -341,11 +364,13 @@ pgfSweaveRuncode <- function(object, chunk, options) {
   else
     lastshown <- srcline - 1
   thisline <- 0
-  
+
   for(nce in 1:length(chunkexps)){
     ce <- chunkexps[[nce]]
+
     if (nce <= length(srcrefs) && !is.null(srcref <- srcrefs[[nce]])) {
         if (options$expand) {
+            # expand references to other chunks
           srcfile <- attr(srcref, "srcfile")
           showfrom <- srcref[1]
           showto <- srcref[3]
@@ -355,32 +380,35 @@ pgfSweaveRuncode <- function(object, chunk, options) {
           showto <- srclines[srcref[3]]
         }
         dce <- getSrcLines(srcfile, lastshown+1, showto)
-            # replace the comment identifiers
-        if(options$tidy){
-          dce <- gsub(sprintf("%s = \"|%s\"", getOption("begin.comment"),
-              getOption("end.comment")), "", dce)
-              # replace tabs with spaces for better looking output
-          dce <- gsub("\\\\t", "    ", dce)
-        }
-            # replace leading lines with #line from 2.12.0
+
+        if(options$tidy) dce <- unmask.source(dce)
+
         leading <- showfrom-lastshown
         lastshown <- showto
         srcline <- srclines[srcref[3]]
-        while (length(dce) && length(grep("^[ \\t]*$", dce[1]))) {
+        
+          # Remove blank lines at head of chunk
+        while (length(dce) && length(grep("^[[:blank:]]*$", dce[1]))) {
           dce <- dce[-1]
           leading <- leading - 1
         }
+
     } else {
+
       dce <- 
       if(options$tidy){
-        deparse2(ce, width.cutoff = 0.75*getOption("width"))
+        deparse.tidy(ce, width.cutoff = 0.75 * getOption("width"))
       }else{
-        deparse(ce, width.cutoff = 0.75*getOption("width"))
+        deparse(ce, width.cutoff = 0.75 * getOption("width"))
       }
+
       leading <- 1
+
     }
+
     if(object$debug)
       cat("\nRnw> ", paste(dce, collapse="\n+  "),"\n")
+
     if(options$echo && length(dce)){
       if(!openSinput & !options$highlight){
         if(!openSchunk){
@@ -393,15 +421,19 @@ pgfSweaveRuncode <- function(object, chunk, options) {
           openSinput <- TRUE
       }
 
-         # Code highlighting stuff
+         # Actual printing of chunk code
       if(options$highlight){
-  
-        if(length(grep("^[[:space:]]*$",dce)) >= 1){
-            # for blank lines for which parser throws an error 
-          cat(translator_latex(paste(getOption("prompt"),'\n', sep="")), 
+
+          # print highlighted output, treat blank lines specially
+        if(length(grep("^[[:space:]]*$",dce)) >= 1 & length(dce) == 1){
+          
+            # for blank lines for which parser throws an error
+          cat(translator_latex(paste(getOption("prompt"),'\n', sep="")),
             file=chunkout, append=TRUE, sep="")
           cat(newline_latex(),file=chunkout, append=TRUE)
+          
         }else{
+
           if(nce == 1)
             cat(newline_latex(),file=chunkout, append=TRUE)
           highlight(parser.output=parser(text=dce),
@@ -409,97 +441,97 @@ pgfSweaveRuncode <- function(object, chunk, options) {
             output = chunkout, showPrompts=TRUE,final.newline = TRUE)
               # highlight doesnt put in an ending newline for some reason
           cat(newline_latex(),file=chunkout, append=TRUE)
+          
         }
 
       }else{
-        cat("\n",paste(getOption("prompt"), dce[1:leading], sep="", 
+          # regular output, may be tidy'd or not
+        cat("\n",paste(getOption("prompt"), dce[1:leading], sep="",
           collapse="\n"), file=chunkout, append=TRUE, sep="")
         if (length(dce) > leading)
-          cat("\n", paste(getOption("continue"), dce[-(1:leading)], sep="", 
+          cat("\n", paste(getOption("continue"), dce[-(1:leading)], sep="",
             collapse="\n"), file=chunkout, append=TRUE, sep="")
+            
       }
+      
       linesout[thisline + 1:length(dce)] <- srcline
       thisline <- thisline + length(dce)
     }
-  
-    ## tmpcon <- textConnection("output", "w")
-    ## avoid the limitations (and overhead) of output text
-    ## connections
-    tmpcon <- file()
-    sink(file=tmpcon)
-    err <- NULL
-    
-    ## [RDP] change this line to use my EvalWithOpt function
-    if(options$eval) err <- pgfSweaveEvalWithOpt(ce, options)
-    ## [CWB] added another output specifying if the code chunk
-    ##     was changed or not. This was an ititial attempt to 
-    ##     improve cacheSweave''s  recognition of chages in a
-    ##     code chunk though it is defunct now.
-    chunkChanged <- FALSE#err$chunkChanged
-    err <- err$err
-    ## [CWB] end
-    ## [RDP] end change
-  
-    cat("\n") # make sure final line is complete
-    sink()
-    output <- readLines(tmpcon)
-    close(tmpcon)
-    ## delete empty output
-    if(length(output)==1 & output[1]=="") output <- NULL
-  
-    RweaveTryStop(err, options)
-  
-    if(object$debug)
-      cat(paste(output, collapse="\n"))
-  
-    if(length(output)>0 & (options$results != "hide")){
-      if(openSinput){
-        cat("\n\\end{Sinput}\n", file=chunkout,append=TRUE)
-        linesout[thisline + 1:2] <- srcline
-        thisline <- thisline + 2
-        openSinput <- FALSE
-      }
-      
-      if(options$results=="verbatim"){
-        if(!openSchunk){
-          cat("\\begin{Schunk}\n",file=chunkout, append=TRUE)
+
+      # do not evaluate empty expressions, these may occur when tidy=T
+    if(length(ce) > 0){
+
+        ## tmpcon <- textConnection("output", "w")
+        ## avoid the limitations (and overhead) of output text connections
+      tmpcon <- file()
+      sink(file=tmpcon)
+      err <- NULL
+
+      if(options$eval) err <- cacheSweaveEvalWithOpt(ce, options)
+      ## [RDP] end change
+
+      cat("\n") # make sure final line is complete
+      sink()
+      output <- readLines(tmpcon)
+      close(tmpcon)
+
+      ## delete empty output
+      if(length(output)==1 & output[1]=="") output <- NULL
+
+      RweaveTryStop(err, options)
+
+      if(object$debug)
+        cat(paste(output, collapse="\n"))
+
+      if(length(output)>0 & (options$results != "hide")){
+        if(openSinput){
+          cat("\n\\end{Sinput}\n", file=chunkout,append=TRUE)
+          linesout[thisline + 1:2] <- srcline
+          thisline <- thisline + 2
+          openSinput <- FALSE
+        }
+
+        if(options$results=="verbatim"){
+          if(!openSchunk){
+            cat("\\begin{Schunk}\n",file=chunkout, append=TRUE)
+            linesout[thisline + 1] <- srcline
+            thisline <- thisline + 1
+            openSchunk <- TRUE
+          }
+          cat("\\begin{Soutput}\n",file=chunkout, append=TRUE)
           linesout[thisline + 1] <- srcline
           thisline <- thisline + 1
-          openSchunk <- TRUE
         }
-        cat("\\begin{Soutput}\n",file=chunkout, append=TRUE)
-        linesout[thisline + 1] <- srcline
-        thisline <- thisline + 1
-      }
-    
-      output <- paste(output,collapse="\n")
-      
-      if(options$strip.white %in% c("all", "true")){
-        output <- sub("^[[:space:]]*\n", "", output)
-        output <- sub("\n[[:space:]]*$", "", output)
-        if(options$strip.white=="all")
-          output <- sub("\n[[:space:]]*\n", "\n", output)
-      }
-      
-      cat(output, file=chunkout, append=TRUE)
-      count <- sum(strsplit(output, NULL)[[1]] == "\n")
-      
-      if (count > 0) {
-        linesout[thisline + 1:count] <- srcline
-        thisline <- thisline + count
-      }
-    
-      remove(output)
-    
-      if(options$results=="verbatim"){
-        cat("\n\\end{Soutput}\n", file=chunkout, append=TRUE)
-        linesout[thisline + 1:2] <- srcline
-        thisline <- thisline + 2
+
+        output <- paste(output,collapse="\n")
+
+        if(options$strip.white %in% c("all", "true")){
+          output <- sub("^[[:space:]]*\n", "", output)
+          output <- sub("\n[[:space:]]*$", "", output)
+          if(options$strip.white=="all")
+            output <- sub("\n[[:space:]]*\n", "\n", output)
+        }
+
+        cat(output, file=chunkout, append=TRUE)
+        count <- sum(strsplit(output, NULL)[[1]] == "\n")
+
+        if (count > 0) {
+          linesout[thisline + 1:count] <- srcline
+          thisline <- thisline + count
+        }
+
+        remove(output)
+
+        if(options$results=="verbatim"){
+          cat("\n\\end{Soutput}\n", file=chunkout, append=TRUE)
+          linesout[thisline + 1:2] <- srcline
+          thisline <- thisline + 2
+        }
       }
     }
   }
 
-  if(options$highlight)
+  if(options$highlight || options$tidy)
     cat("\n", file=chunkout, append=TRUE)
 
   if(openSinput){
@@ -514,41 +546,39 @@ pgfSweaveRuncode <- function(object, chunk, options) {
     thisline <- thisline + 1
   }
 
-  #put in an extra newline before the output for good measure
-  cat("\n", file=chunkout, append=TRUE)
-
 
   if(is.null(options$label) & options$split)
     close(chunkout)
 
   if(options$split & options$include){
-    cat("\\input{", chunkprefix, "}\n", sep="", 
+    cat("\\input{", chunkprefix, "}\n", sep="",
       file=object$output, append=TRUE)
     linesout[thisline + 1] <- srcline
     thisline <- thisline + 1
   }
 
   if(options$fig && options$eval){
+
     ## [CWB] adding checking for external options
     pdfExists <- file.exists(paste(chunkprefix, "pdf", sep="."))
     epsExists <- file.exists(paste(chunkprefix, "eps", sep="."))
-    
+
     if(options$eps & !options$pgf & !options$tikz){
-      if(!options$external | (chunkChanged | !epsExists) ){
-        
+      if(!options$external | !epsExists ){
+
           # [CWB] the useKerning option was added in R 2.9.0
-          # so check for the R version and set the 
-          # useKerning option to false if 2.8.x or less is 
-          # used. 
+          # so check for the R version and set the
+          # useKerning option to false if 2.8.x or less is
+          # used.
         if(getRversion() < "2.9.0")
-          grDevices::postscript(file = paste(chunkprefix, "eps", sep="."), 
-            width=options$width, height=options$height, 
+          grDevices::postscript(file = paste(chunkprefix, "eps", sep="."),
+            width=options$width, height=options$height,
             paper="special", horizontal=FALSE)
-        else 
-          grDevices::postscript(file = paste(chunkprefix, "eps", sep="."), 
-            width=options$width, height=options$height, 
+        else
+          grDevices::postscript(file = paste(chunkprefix, "eps", sep="."),
+            width=options$width, height=options$height,
             paper="special",horizontal=FALSE, useKerning=FALSE)
-    
+
         err <- try({
           SweaveHooks(options, run=TRUE)
           eval(chunkexps, envir=.GlobalEnv)
@@ -558,11 +588,11 @@ pgfSweaveRuncode <- function(object, chunk, options) {
       }
     }
     if(options$pdf & !options$pgf & !options$tikz){
-      if(!options$external | (chunkChanged | !pdfExists) ){
+      if(!options$external | !pdfExists ){
         grDevices::pdf(file=paste(chunkprefix, "pdf", sep="."),
-          width=options$width, height=options$height, 
+          width=options$width, height=options$height,
           version=options$pdf.version, encoding=options$pdf.encoding)
-    
+
         err <- try({
           SweaveHooks(options, run=TRUE)
           eval(chunkexps, envir=.GlobalEnv)
@@ -571,58 +601,57 @@ pgfSweaveRuncode <- function(object, chunk, options) {
         if(inherits(err, "try-error")) stop(err)
       }
     }
-    
+
     #############################################
-    ## [CWB] Here are the new options related to graphics output 
-    ## pgf: uses eps2pgf with the `-m directcopy` option to create 
-    ##    a pgf file in which the text is interperted as a tex 
-    ##    string 
-    ## 
-    ## external: uses the external feature in 
+    ## [CWB] Here are the new options related to graphics output
+    ## pgf: uses eps2pgf with the `-m directcopy` option to create
+    ##    a pgf file in which the text is interperted as a tex
+    ##    string
+    ##
+    ## external: uses the external feature in
     ##   the TeX package pgf to include the compiled pdf file if
     ##   available
     ##
-    
+
     if(options$pgf){
-      if(chunkChanged | !pdfExists){
-        
+      if( !pdfExists ){
+
           # [CWB] the useKerning option was added in R 2.9.0
-          # so check for the R version and set the 
-          # useKerning option to false if 2.8.x or less is 
-          # used. 
+          # so check for the R version and set the
+          # useKerning option to false if 2.8.x or less is
+          # used.
         if(getRversion() < "2.9.0")
           grDevices::postscript(file = paste(chunkprefix, "eps", sep="."),
-            width=options$width, height=options$height, 
+            width=options$width, height=options$height,
             paper="special", horizontal=FALSE)
-        else 
-          grDevices::postscript(file = paste(chunkprefix, "eps", sep="."), 
-            width=options$width, height=options$height, 
+        else
+          grDevices::postscript(file = paste(chunkprefix, "eps", sep="."),
+            width=options$width, height=options$height,
             paper="special",horizontal=FALSE, useKerning=FALSE)
-    
+
         err <- try({
           SweaveHooks(options, run=TRUE)
           eval(chunkexps, envir=.GlobalEnv)
           })
         grDevices::dev.off()
         if(inherits(err, "try-error")) stop(err)
-    
+
         eps2pgf <- system.file(package='pgfSweave')
         eps2pgf <- paste('java -jar ',eps2pgf,
           '/java/eps2pgf/eps2pgf.jar -m directcopy',sep='')
-                    
+
         cat('Generating pgf file from:',paste(chunkprefix, "eps",sep="."),'\n')
         err <- try(system(paste(eps2pgf,paste(chunkprefix, "eps",sep="."))))
-                    
+
         if(inherits(err, "try-error")) stop(err)
       }
     }
     if(options$tikz){
-      if(chunkChanged | ifelse(options$tex.driver == "latex", 
-          !epsExists,!pdfExists)){
-        tikzDevice::tikz(file=paste(chunkprefix, "tikz", sep="."), 
-          width=options$width, height=options$height, 
+      if(ifelse(options$tex.driver == "latex", !epsExists, !pdfExists)){
+        tikzDevice::tikz(file=paste(chunkprefix, "tikz", sep="."),
+          width=options$width, height=options$height,
           sanitize=options$sanitize)
-    
+
         err <- try({
           SweaveHooks(options, run=TRUE)
           eval(chunkexps, envir=.GlobalEnv)
@@ -632,44 +661,51 @@ pgfSweaveRuncode <- function(object, chunk, options) {
       }
     }
     
-    if(options$external){
-      if( chunkChanged | ifelse(options$tex.driver == "latex", 
-          !epsExists,!pdfExists) && (!options$pdf && !options$eps)){
-      
+      # External option, write commands to externalize graphics if the 
+      # graphic file does not exist
+    if( options$external ){
+      if(ifelse(options$tex.driver == "latex", !epsExists, !pdfExists) && 
+         (!options$pdf && !options$eps)){
+
         shellFile <- object[["shellFile"]]
         tex.driver <- options[["tex.driver"]]
-        
-        cat(tex.driver, ' --jobname=', chunkprefix,' ', 
+
+        cat(tex.driver, ' --jobname=', chunkprefix,' ',
           object[["srcfileName"]], '\n', sep='',  file=shellFile, append=TRUE)
         if(tex.driver == "latex"){
-          cat('dvipdf ',paste(chunkprefix,'dvi',sep='.'), '\n', sep='', 
+          cat('dvipdf ',paste(chunkprefix,'dvi',sep='.'), '\n', sep='',
             file=shellFile, append=TRUE)
           cat('pdftops -eps ',paste(chunkprefix,'pdf',sep='.'),
             '\n', sep='', file=shellFile, append=TRUE)
         }
       }
     }
+      # Write the extrnalization commands if the user does not want 
+      # to do it themselves already
     if(options$include && options$external) {
       cat("\n\\beginpgfgraphicnamed{",chunkprefix,"}\n",sep="",
         file=object$output, append=TRUE)
       linesout[thisline + 1] <- srcline
       thisline <- thisline + 1
-    } 
+    }
+      # Write the includegraphics command for eps or pdf 
+      # only if we are not useing pgf or tikz
     if(options$include && !options$pgf && !options$tikz && !options$external) {
       cat("\\includegraphics{", chunkprefix, "}\n", sep="",
         file=object$output, append=TRUE)
       linesout[thisline + 1] <- srcline
       thisline <- thisline + 1
     }
+      # input statements for tikz and pgf
     if(options$include && (options$pgf || options$tikz)) {
-      #if tikz is TRUE or both tikz and pgf are true, 
-      # use tikz, otherwise use pgf
+      #if tikz takes precident over pgf option
       suffix <- ifelse(options$tikz,'tikz','pgf')
       cat("\\input{", paste(chunkprefix,suffix,sep='.'),
         "}\n", sep="", file=object$output, append=TRUE)
       linesout[thisline + 1] <- srcline
       thisline <- thisline + 1
     }
+      # ending externalization command
     if(options$include && options$external) {
       cat("\\endpgfgraphicnamed\n",sep="",file=object$output, append=TRUE)
       linesout[thisline + 1] <- srcline
